/* FTP Hub Client for OS/2
   Michael Bilow, 1996-Jan-13

   Latest revision: 1997-Mar-21

   Usage:
   FTPHUBC host userid password remnet/remnode remin remout locin locout

   Example:
   FTPHUBC ftp.sstar.com 323.107 pass 396/1 in out c:\bink\files c:\bink\out
 */


/*****************************************************************************
 * Set up initial exception handling
 *****************************************************************************/
signal on error name Abort1
signal on halt name Abort1

/*****************************************************************************
 * Initialize the log file
 *****************************************************************************/
logfile = 'c:\bink\logs\ftphubc.log'
call lineout logfile, ' '
logline = 'Starting (' || time('E') || ' seconds elapsed)'
call log 

/*****************************************************************************
 * Bring in system DLLs
 *****************************************************************************/
lfSys = RxFuncQuery("SysLoadFuncs")
if lfSys \= 0 then lfSys = RxFuncAdd("SysLoadFuncs", "RexxUtil", "SysLoadFuncs")

lfFTP = RxFuncQuery("FtpLoadFuncs")
if lfFTP \= 0 then lfFTP = RxFuncAdd("FtpLoadFuncs", "rxFtp", "FtpLoadFuncs")

if ((lfSys \= 0) | (lfFTP \= 0)) then
	do
		logline = 'ERROR: Library load failed'
		call log
		signal Abort1
	end
else
	do
		call SysLoadFuncs
		call FtpLoadFuncs
		logline = 'Libraries loaded'
		call log
	end

/*****************************************************************************
 * Read and parse remote system information from command line
 *****************************************************************************/
parse arg host userid password remnet'/'remnode remin remout locin locout

logline = 'Host: ' || host || ', User: ' || userid
call log
if ((length(host)  = 0) | (length(userid) = 0)) then
	do
		logline = 'ERROR: No host or userid specified'
		call log
		signal Abort1
	end

remhex = d2x(remnet, 4) || d2x(remnode, 4)
logline = 'Remote -- hex: ' || remhex || ', inbound: ' || remin || ', outbound: ' || remout
call log
if ((length(remnet) = 0) | (length(remnode) = 0) | (length(remin) = 0) | (length(remout) = 0)) then
	do
		logline = 'ERROR: No remote information specified'
		call log
		signal Abort1
	end
call strip remin,  'T', '\'
call strip remout, 'T', '\'


logline = 'Local  -- inbound: ' || locin || ', outbound: ' || locout
call log
if ((length(locin)  = 0) | (length(locout) = 0)) then
	do
		logline = 'ERROR: No local information specified'
		call log
		signal Abort1
	end
call strip locin,  'T', '\'
call strip locout, 'T', '\'

/*****************************************************************************
 * Write a dummy busy file for copying later
 *****************************************************************************/
/* busyfile = strip(left(userid, 8)) || '.bsy' */
busyfile = ihub.bsy
call lineout busyfile, userid || ' ' || date('O') || ' ' || time('N'), 1
call stream busyfile, 'C', 'close'
logline = 'Remote busy flag file name: ' || busyfile
call log

/*****************************************************************************
 * See if we are busy locally
 *****************************************************************************/
/* This is kind of dangerous, since the check and creation of the flag is not
 * guaranteed to be atomic.  However, there is no way to fix this strictly
 * within the REXX language, so we live with it for now.
 */
locbsy = locout || '\' || remhex || '.bsy'
logline = 'Local busy flag file name: ' || locbsy
call log

call SysFileTree locbsy, 'flst.', 'FO'
if flst.0 = 0 then
	do
		r = lineout(locbsy, date('O') || ' ' || time('M'), 1)
	    	call stream locbsy, 'c', 'close'
	end
else
	r = -1

if r = 0 then
	do
		/* change exception handling to account for local busy flag */
		signal on error name Abort2
		signal on halt name Abort2

		logline = 'Asserted local busy flag'
		call log
	end
else
	do
		logline = 'ERROR: Unable to assert local busy flag; already busy?'
		call log
		signal Abort1
	end

/*****************************************************************************
 * Bring up the network and test connectivity
 *****************************************************************************/
do 0
	r = FtpPing(host, 64)
	logline = "Testing network link... " || r
	call log
	if r \= 'PINGREPLY' then leave
end
if ((r = 'PINGREPLY') | /* (r = 'PINGSEND') | */ (r = 'PINGHOST')) then
	do
		logline = 'ERROR(' || FTPERRNO || '): No connectivity; ping ' || host || ' failed'
		call log
		signal Abort2
	end
else
	do
		logline = 'Ping to ' || host || ' required ' || r || ' ms'
		call log
	end

/*****************************************************************************
 * Open FTP channel
 *****************************************************************************/
if FtpSetUser(host, userid, password) = 1 then
	do
		logline = 'Opened channel to ' || host
		call log
		if FtpSys(remsys) = 0 then
			do
				logline = 'Remote is running ' || remsys
				call log
			end
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure opening channel to ' || host
		call log
		signal Abort2
	end

/*****************************************************************************
 * Set FTP channel to binary mode
 *****************************************************************************/
if FtpSetBinary("Binary") = 1 then
	do
		logline = 'Channel set to binary mode'
		call log
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure setting channel to binary mode'
		call log
		signal Abort2
	end

/*****************************************************************************
 * Save our home directory for busy flag maintenance
 *****************************************************************************/
if FtpPwd(remhomestr) = 0 then
	do
		parse var remhomestr remhome ' '
		remhome = strip(remhome, 'Both', '"')	/* funny FTP server */
		logline = 'Remote home directory: ' || remhome
		call log
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure recording remote home directory'
		call log
		signal Abort2
	end

/*****************************************************************************
 * Write the busy flag on remote
 *****************************************************************************/
if FtpPut(busyfile, busyfile) = 0 then
	do
		logline = 'Asserting busy on remote'
		call log
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure asserting busy on remote'
		call log
		signal Abort2
	end

/*****************************************************************************
 * Handle files outbound from us
 *****************************************************************************/
lochlo = locout || '\' || remhex || '.hlo'
call SysFileTree lochlo, 'flst.', 'FO'
if flst.0 = 0 then
	do
		logline = 'Nothing going outbound locally'
		call log
	end
else
	do		/* We have something going out */
		logline = 'Outbound hlo ' || lochlo || ' found'
		call log

		if FtpChDir(remin) = 0 then
			do	/* We are in the remote inbound directory */
				logline = 'Changed to remote inbound directory: ' || remin
				call log

				do forever while (lines(lochlo) = 1)

					fileout = linein(lochlo)

					/* Handle Binkley truncate and delete flags */
					fileoutflag = left(fileout, 1)
					if ((fileoutflag = '#') | (fileoutflag = '^')) then
						fileout = substr(fileout, 2)

					/* Handle special case where file in HLO does not exist */
					call SysFileTree fileout, 'flsttemp.', 'F'
					if flsttemp.0 = 0 then iterate

					/* read the hlo and send these files */
					if FtpPut(fileout, filespec("name", fileout)) = 0 then
						do
							logline = 'Put ' || fileout || ' successfully'
							call log

							if ((fileoutflag = '^') | (fileoutflag = '#')) then
								do
									if SysFileDelete(fileout) = 0 then
										do
											if fileoutflag = '#' then
												do
													call stream fileout, 'C', 'open write'
													call stream fileout, 'C', 'close'
													logline = 'Truncating ' || fileout
												end
											else
												logline = 'Deleting ' || fileout
											call log
										end
									else
										do
											logline = 'ERROR: Cannot delete/truncate ' || fileoutflag || fileout
											call log
										end
								end
						end
					else
						do
							call SysFileTree fileout, 'flsttemp.', 'F'
							if flsttemp.0 > 0 then
							do
								logline = 'Failing file information: ' || flsttemp.1
								call log
							end
							logline = 'ERROR(' || FTPERRNO || '): Put ' || fileout || ' failed'
							call log
							signal Abort2
						end
				end

				/* If we're here, all files in the hlo have been handled */
				logline = 'All outbound files have been sent'
				call log

				call stream lochlo, 'c', 'close'
				if SysFileDelete(lochlo) = 0 then
					logline = 'Deleted local hlo file ' || lochlo
				else
						logline = 'ERROR: Unable to delete local hlo ' || lochlo
				call log

				if FtpChDir(remhome) = 0 then
					do
						logline = 'Returning to remote home directory: ' || remhome
						call log
					end
				else
					do
						logline = 'ERROR(' || FTPERRNO || '): Failure returning to remote home directory'
						call log
						signal Abort2
					end
			end
		else
			do
				logline = 'ERROR(' || FTPERRNO || '): Unable to change to remote inbound directory ' || remin
				call log
			end
	end		/* We have something going out */

/*****************************************************************************
 * Handle files inbound to us
 *****************************************************************************/
if FtpChDir(remout) = 0 then
	do
		logline = 'Changed to remote outbound directory: ' || remout
		call log

		liststat = FtpLs("*.*", "flst.")	/* use '*.*' instead of '*' for DOS ftpd */
		call sortall
		call sorttic
		say 'File count: ' || flst.0
		do i = 1 to flst.0
			say 'File ' || i || ': ' || flst.i
		end

		if ((liststat = -1) | ((liststat = 0) & (flst.0 = 0))) then
			do
				logline = 'No files in remote outbound for us to pick up'
				call log
			end
		else
			do i = 1 to flst.0
				if left(flst.i, 6) = 'total:' then	/* funny FTP server */
					do
						if i = 1 then
							do
								logline = 'No files in remote outbound for us to pick up'
								call log
							end
						leave
					end
				logline = 'Starting retrieve file ' || flst.i || '...'
				call log
				if FtpGet(locin || '\' || flst.i, flst.i) = 0 then
					do
						logline = 'Completed retrieve file ' || flst.i
						call log
						if FtpDelete(flst.i) = 0 then
							do
								logline = 'Deleted remote file ' || flst.i || ' after retrieve'
								call log
							end
					end
				else
					do
						logline = 'ERROR(' || FTPERRNO || '): Retrieving file ' || flst.i
						call log
					end
			end

		if FtpChDir(remhome) = 0 then
			do
				logline = 'Returning to remote home directory: ' || remhome
				call log
			end
		else
			do
				logline = 'ERROR(' || FTPERRNO || '): Failure returning to remote home directory ' || remhom
				call log
				signal Abort2
			end
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Unable to change to remote outbound directory ' || remout
		call log
	end

/*****************************************************************************
 * Change remote directory back to home
 *****************************************************************************/
if FtpChDir(remhome) = 0 then
	do
		logline = 'Returning to remote home directory: ' || remhome
		call log
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure returning to remote home directory'
		call log
		signal Abort2
	end

/*****************************************************************************
 * Delete the busy flag on remote
 *****************************************************************************/
if FtpDelete(busyfile) = 0 then
	do
		logline = 'Deasserting busy on remote'
		call log
	end
else
	do
		logline = 'ERROR(' || FTPERRNO || '): Failure deasserting busy on remote'
		call log
		signal Abort2
	end

/*****************************************************************************
 * Termination sequences
 *****************************************************************************/
logline = 'Beginning normal termination sequence'
call log
call SysFileDelete locbsy
signal NormalTerm

/*****************************************************************************
 * Abnormal termination routine
 *****************************************************************************/
Abort2:
call SysFileDelete locbsy
Abort1:
logline = 'ABORT: Last line before logoff: ' || SIGL
call log
NormalTerm:

/*****************************************************************************
 * Log off and close; this point should always be reached unconditionally
 *****************************************************************************/
logline = 'Logoff returned ' || FtpLogOff()
call log

/*****************************************************************************
 * Timestamp termination and skip trailing line in log file
 *****************************************************************************/
logline = 'Ending (' || time('E') || ' seconds elapsed)'
call log

/*****************************************************************************
 * Exit from main routine
 *****************************************************************************/
exit


/*****************************************************************************
 *
 * Subroutines
 *
 *****************************************************************************/

/*****************************************************************************
 * LOG: Make log entries
 *****************************************************************************/
log: procedure expose logfile logline
call lineout logfile, date('O') || ' ' || time('N') || ' FTPH ' || logline
say logline
return

/*****************************************************************************
 * SORTTIC: Sort stem variable so *.TIC files are last
 *****************************************************************************/
sorttic: procedure expose flst.

	last = flst.0
	swapped = 1

	do while(swapped == 1)

		swapped = 0
		if(last < 2) then leave

		do i = 1 to last-1

			t1 = flst.i
			t1 = right(t1, 4)
			parse upper var t1 t1

			j = i+1
			t2 = flst.j
			t2 = right(t2, 4)
			parse upper var t2 t2

			if((t2 <> '.TIC') & (t1 == '.TIC')) then
				do
					temp = flst.i
					flst.i = flst.j
					flst.j = temp

					swapped = 1
				end

		end
		last = last - 1
	end
return

/*****************************************************************************
 * SORTALL: Sort stem variable in ASCII order
 *****************************************************************************/
sortall: procedure expose flst.

	last = flst.0
	swapped = 1

	do while(swapped == 1)

		swapped = 0
		if(last < 2) then leave

		do i = 1 to last-1

			t1 = flst.i
			parse upper var t1 t1

			j = i+1
			t2 = flst.j
			parse upper var t2 t2

			if(t2 < t1) then
				do
					temp = flst.i
					flst.i = flst.j
					flst.j = temp

					swapped = 1
				end

		end
		last = last - 1
	end
return


